module Goldie
  class Item < ActiveRecord::Base
    attr_accessible :title, :weight, :weight_completed, :archived

    validates :title, presence: true
    validates :weight, :weight_completed, numericality: { only_integer: true }
    validate :weight_completed_must_be_between_zero_and_weight

    scope :archived, where(archived: true)
    scope :active, where(archived: false)
    scope :done, where("items.weight_completed == items.weight")
    scope :not_done, where("items.weight_completed != items.weight")
    scope :latest, order("items.created_at DESC")

    def done?
      weight_completed == weight
    end

    def done!
      update_attributes!(weight_completed: weight)
    end

    def not_done!
      update_attributes!(weight_completed: 0)
    end

    def archive!
      update_attributes!(:archived, true)
    end

    def unarchive!
      update_attributes!(:archived, false)
    end

    def percent_done
      weight_completed * 100.0 / weight
    end

    private

    def weight_completed_must_be_between_zero_and_weight
      unless (0..weight).include? weight_completed
        errors.add(:weight_completed, "must be between zero and weight")
      end
    end
  end
end
